@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: BSD-2-Clause */
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2021 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2019 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -81,14 +81,6 @@
 #define	OT_BITFLAG		(1 << 27)
 #define	OT_RESERVED		(1 << 28)
 
-#define	DHC_REQ(r, n, o) \
-	(has_option_mask((r), (o)) && !has_option_mask((n), (o)))
-
-#define DHC_REQOPT(o, r, n)						  \
-	(!((o)->type & OT_NOREQ) &&					  \
-	 ((o)->type & OT_REQUEST || has_option_mask((r), (o)->option)) && \
-	  !has_option_mask((n), (o)->option))
-
 struct dhcp_opt {
 	uint32_t option; /* Also used for IANA Enterpise Number */
 	int type;
@@ -136,11 +128,6 @@ void dhcp_envoption(struct dhcpcd_ctx *,
     const uint8_t *, size_t, struct dhcp_opt **),
     const uint8_t *od, size_t ol);
 void dhcp_zero_index(struct dhcp_opt *);
+size_t dhcp_read_lease_fd(int, void **);
 
-ssize_t dhcp_readfile(struct dhcpcd_ctx *, const char *, void *, size_t);
-ssize_t dhcp_writefile(struct dhcpcd_ctx *, const char *, mode_t,
-    const void *, size_t);
-int dhcp_filemtime(struct dhcpcd_ctx *, const char *, time_t *);
-int dhcp_unlink(struct dhcpcd_ctx *, const char *);
-size_t dhcp_read_hwaddr_aton(struct dhcpcd_ctx *, uint8_t **, const char *);
 #endif
