@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: BSD-2-Clause */
 /*
  * Solaris interface driver for dhcpcd
  * Copyright (c) 2016-2019 Roy Marples <roy@marples.name>
@@ -76,7 +77,7 @@ extern int getallifaddrs(sa_family_t, struct ifaddrs **, int64_t);
 #ifndef RT_ROUNDUP
 #define RT_ROUNDUP(a)                                                        \
        ((a) > 0 ? (1 + (((a) - 1) | (sizeof(int32_t) - 1))) : sizeof(int32_t))
-#define RT_ADVANCE(x, n) ((x) += RT_ROUNDUP(salen((n))))
+#define RT_ADVANCE(x, n) ((x) += RT_ROUNDUP(sa_len((n))))
 #endif
 
 #define COPYOUT(sin, sa) do {						      \
@@ -91,7 +92,7 @@ extern int getallifaddrs(sa_family_t, struct ifaddrs **, int64_t);
 		    (sa))->sin6_addr;					      \
 	} while (0)
 
-#define COPYSA(dst, src) memcpy((dst), (src), salen((src)))
+#define COPYSA(dst, src) memcpy((dst), (src), sa_len((src)))
 
 struct priv {
 #ifdef INET6
@@ -399,22 +400,6 @@ if_getifaddrs(struct ifaddrs **ifap)
 	return 0;
 }
 
-static int
-salen(const struct sockaddr *sa)
-{
-
-	switch (sa->sa_family) {
-	case AF_LINK:
-		return sizeof(struct sockaddr_dl);
-	case AF_INET:
-		return sizeof(struct sockaddr_in);
-	case AF_INET6:
-		return sizeof(struct sockaddr_in6);
-	default:
-		return sizeof(struct sockaddr);
-	}
-}
-
 static void
 if_linkaddr(struct sockaddr_dl *sdl, const struct interface *ifp)
 {
@@ -540,7 +525,7 @@ if_route0(struct dhcpcd_ctx *ctx, struct rtm *rtmsg,
 	 * This includes subnet/prefix routes. */
 
 #define ADDSA(sa) do {							\
-		sl = salen((sa));					\
+		sl = sa_len((sa));					\
 		memcpy(bp, (sa), sl);					\
 		bp += RT_ROUNDUP(sl);					\
 	} while (/* CONSTCOND */ 0)
@@ -1078,9 +1063,7 @@ if_handlelink(struct dhcpcd_ctx *ctx)
 		return -1;
 	if (len == 0)
 		return 0;
-	if ((size_t)len < offsetof(struct rt_msghdr, rtm_index) ||
-	    len < rtm.hdr.rtm_msglen)
-	{
+	if (len < rtm.hdr.rtm_msglen) {
 		errno = EINVAL;
 		return -1;
 	}
@@ -1329,10 +1312,10 @@ if_plumb(int cmd, const struct dhcpcd_ctx *ctx, int af, const char *ifname)
 
 #ifdef INET
 static int
-if_walkrt(struct dhcpcd_ctx *ctx, char *data, size_t len)
+if_walkrt(struct dhcpcd_ctx *ctx, rb_tree_t *routes, char *data, size_t len)
 {
 	mib2_ipRouteEntry_t *re, *e;
-	struct rt rt;
+	struct rt rt, *rtn;
 	char ifname[IF_NAMESIZE];
 	struct in_addr in;
 
@@ -1357,7 +1340,6 @@ if_walkrt(struct dhcpcd_ctx *ctx, char *data, size_t len)
 		}
 
 		memset(&rt, 0, sizeof(rt));
-		rt.rt_dflags |= RTDF_INIT;
 		in.s_addr = re->ipRouteDest;
 		sa_in_init(&rt.rt_dest, &in);
 		in.s_addr = re->ipRouteMask;
@@ -1370,10 +1352,17 @@ if_walkrt(struct dhcpcd_ctx *ctx, char *data, size_t len)
 		rt.rt_mtu = re->ipRouteInfo.re_max_frag;
 		if_octetstr(ifname, &re->ipRouteIfIndex, sizeof(ifname));
 		rt.rt_ifp = if_find(ctx->ifaces, ifname);
-		if (if_finishrt(ctx, &rt) == -1)
+		if (if_finishrt(ctx, &rt) == -1) {
 			logerr(__func__);
-		else
-			rt_recvrt(RTM_ADD, &rt, 0);
+			continue;
+		}
+		if ((rtn = rt_new(rt.rt_ifp)) == NULL) {
+			logerr(__func__);
+			break;
+		}
+		memcpy(rtn, &rt, sizeof(*rtn));
+		if (rb_tree_insert_node(routes, rtn) != rtn)
+			rt_free(rtn);
 	} while (++re < e);
 	return 0;
 }
@@ -1381,10 +1370,10 @@ if_walkrt(struct dhcpcd_ctx *ctx, char *data, size_t len)
 
 #ifdef INET6
 static int
-if_walkrt6(struct dhcpcd_ctx *ctx, char *data, size_t len)
+if_walkrt6(struct dhcpcd_ctx *ctx, rb_tree_t *routes, char *data, size_t len)
 {
 	mib2_ipv6RouteEntry_t *re, *e;
-	struct rt rt;
+	struct rt rt, *rtn;
 	char ifname[IF_NAMESIZE];
 	struct in6_addr in6;
 
@@ -1410,7 +1399,6 @@ if_walkrt6(struct dhcpcd_ctx *ctx, char *data, size_t len)
 		}
 
 		memset(&rt, 0, sizeof(rt));
-		rt.rt_dflags |= RTDF_INIT;
 		sa_in6_init(&rt.rt_dest, &re->ipv6RouteDest);
 		ipv6_mask(&in6, re->ipv6RoutePfxLength);
 		sa_in6_init(&rt.rt_netmask, &in6);
@@ -1419,18 +1407,26 @@ if_walkrt6(struct dhcpcd_ctx *ctx, char *data, size_t len)
 		rt.rt_mtu = re->ipv6RouteInfo.re_max_frag;
 		if_octetstr(ifname, &re->ipv6RouteIfIndex, sizeof(ifname));
 		rt.rt_ifp = if_find(ctx->ifaces, ifname);
-		if (if_finishrt(ctx, &rt) == -1)
+		if (if_finishrt(ctx, &rt) == -1) {
 			logerr(__func__);
-		else
-			rt_recvrt(RTM_ADD, &rt, 0);
+			continue;
+		}
+		if ((rtn = rt_new(rt.rt_ifp)) == NULL) {
+			logerr(__func__);
+			break;
+		}
+		memcpy(rtn, &rt, sizeof(*rtn));
+		if (rb_tree_insert_node(routes, rtn) != rtn)
+			rt_free(rtn);
 	} while (++re < e);
 	return 0;
 }
 #endif
 
 static int
-if_parsert(struct dhcpcd_ctx *ctx, unsigned int level, unsigned int name,
-    int (*walkrt)(struct dhcpcd_ctx *, char *, size_t))
+if_parsert(struct dhcpcd_ctx *ctx, rb_tree_t *routes,
+    unsigned int level, unsigned int name,
+    int (*walkrt)(struct dhcpcd_ctx *, rb_tree_t *, char *, size_t))
 {
 	int			s, retval, code, flags;
 	uintptr_t		buf[512 / sizeof(uintptr_t)];
@@ -1515,7 +1511,7 @@ if_parsert(struct dhcpcd_ctx *ctx, unsigned int level, unsigned int name,
 		 * the next item, so don't move this test higher up
 		 * to avoid the buffer allocation and getmsg calls. */
 		if (req->level == level && req->name == name) {
-			if (walkrt(ctx, databuf.buf, req->len) == -1)
+			if (walkrt(ctx, routes, databuf.buf, req->len) == -1)
 				break;
 		}
 	}
@@ -1528,18 +1524,17 @@ out:
 
 
 int
-if_initrt(struct dhcpcd_ctx *ctx, int af)
+if_initrt(struct dhcpcd_ctx *ctx, rb_tree_t *routes, int af)
 {
 
-	rt_headclear(&ctx->kroutes, af);
 #ifdef INET
 	if ((af == AF_UNSPEC || af == AF_INET) &&
-	    if_parsert(ctx, MIB2_IP,MIB2_IP_ROUTE, if_walkrt) == -1)
+	    if_parsert(ctx, routes, MIB2_IP,MIB2_IP_ROUTE, if_walkrt) == -1)
 		return -1;
 #endif
 #ifdef INET6
 	if ((af == AF_UNSPEC || af == AF_INET6) &&
-	    if_parsert(ctx, MIB2_IP6, MIB2_IP6_ROUTE, if_walkrt6) == -1)
+	    if_parsert(ctx, routes, MIB2_IP6, MIB2_IP6_ROUTE, if_walkrt6) == -1)
 		return -1;
 #endif
 	return 0;
